RELEASE=7

if [ -n "${1:-}" ]; then
    RELEASE="$1"
fi
echo "Setting RELEASE=$RELEASE"

DistType "redhat"
MirrorURL "http://ftp.scientificlinux.org/linux/scientific/${RELEASE}x/\$basearch/os/"

Setup
Bootstrap

#InstallPkgs yum vim-minimal

Cleanup

