# Copyright (c) 2015-2017, Gregory M. Kurtzer. All rights reserved.
# 
# "Singularity" Copyright (c) 2016, The Regents of the University of California,
# through Lawrence Berkeley National Laboratory (subject to receipt of any
# required approvals from the U.S. Dept. of Energy).  All rights reserved.

BootStrap: debootstrap
OSVersion: stable
MirrorURL: http://ftp.us.debian.org/debian/


%runscript
    echo "This is what happens when you run the container..."


%post
    echo "Hello from inside the container"
    apt-get update
    apt-get -y --force-yes install vim

