DistType "debian"
MirrorURL "http://us.archive.ubuntu.com/ubuntu/"
OSVersion "trusty"

Setup
Bootstrap

RunCmd sed -i 's/$/ universe/' /etc/apt/sources.list
RunCmd apt-get update

InstallPkgs vim

Cleanup
