/* 
 * Copyright (c) 2017, SingularityWare, LLC. All rights reserved.
 * 
 * This software is licensed under a 3-clause BSD license.  Please
 * consult LICENSE file distributed with the sources of this project regarding
 * your rights to use or distribute this software.
 * 
 */

#ifndef __SINGULARITY_RUNTIME_NS_NET_H_
#define __SINGULARITY_RUNTIME_NS_NET_H_

extern int _singularity_runtime_ns_net(void);
extern int _singularity_runtime_ns_net_join(void);

#endif /* __SINGULARITY_RUNTIME_NS_NET_H */

